package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderToSapService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel.MesExcelTool;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

@Api(description = "工单主页面")
@RestController
@RequestMapping(MesCommonConstant.MES_YANFEN + "/mesWorkOrder")
public class MesWorkOrderController extends BaseMesController<MesWorkOrder> {

    @Autowired
    private IMesWorkOrderService mesWorkOrderService;

    @Autowired
    private IMesWorkOrderToSapService mesWorkOrderToSapService;

    @PostMapping(value = "/import-mes-work-order")
    @ApiOperation(value = "导入数据")
    public ResultBean importMesWorkOrderExcel(@RequestParam("file") MultipartFile file, @RequestParam("status") Integer type) {
        try {
            MesExcelTool excelTool = new MesExcelTool(entityManager, RedisCacheTool.getImppRedis());
            List<MesWorkOrder> beanList = excelTool.importData(file.getOriginalFilename(), file.getInputStream(), MesWorkOrder.class);

            // 校验导入数据
            validateImport(beanList);
            String userName = AuthUtil.getSessionUser().getUserName();
            String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
            // 导入数据初始化
            for (MesWorkOrder bean : beanList) {
                bean.setWorkOrderType(type);
                ConvertBean.serviceModelInitialize(bean, userName);
                bean.setOrganizeCode(organizeCode);
            }
            baseService.insertBatch(beanList);

            // 导入后
            afterImport(beanList);
            return ResultBean.success("导入成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/doProductReport")
    @ApiOperation(value = "生产报工")
    public ResultBean doProductReport(@RequestBody MesWorkOrder mesWorkOrder) {
        try {
            //新增初始化
            mesWorkOrder.setOrganizeCode(AuthUtil.getOrganizeCode());
            ConvertBean.serviceModelInitialize(mesWorkOrder, AuthUtil.getSessionUser().getUserName());
            mesWorkOrderService.doProductReport(mesWorkOrder);
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException e) {
            return ResultBean.fail(e).build();
        } catch (Exception e) {
            return ResultBean.fail(e);
        }
    }


    @GetMapping(value = "/sendToSap")
    @ApiOperation(value = "发送给sap")
    public ResultBean sendToSap(String organizeCode) {
        try {
            //查询汇总工单数量
            List<MesWorkOrder> mesWorkOrderList = mesWorkOrderService.sumMesWorkOrderQtyToSap(organizeCode);
            if (!CollectionUtils.isEmpty(mesWorkOrderList)){
                mesWorkOrderToSapService.doMesWorkOrder(organizeCode,mesWorkOrderList);
            }
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException e) {
            return ResultBean.fail(e).build();
        } catch (Exception e) {
            return ResultBean.fail(e);
        }
    }



}
