package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;

import cn.estsh.i3plus.ext.mes.api.base.IMesDefectAlarmConfigService;
import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description : 按工厂划分：把 SAP 接口表的数据同步到 MES 业务表中（单据+基础数据）
 * @Reference :
 * @Author : gsz
 * @CreateDate : 2024-05-10 11:03
 * @Modify:
 **/
// 禁止 JOB 并发执行
@DisallowConcurrentExecution
@ApiOperation("缺陷告警配置job")
public class MesDefectAlarmConfigJob extends BaseMesScheduleJob {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesDefectAlarmConfigJob.class);

    private static final long serialVersionUID = 9072058154235836696L;

    @Autowired
    private IMesDefectAlarmConfigService defectAlarmConfigService;

    @Value("${server.port}")
    private String serverPort;
    @Value("${impp.server.ip}")
    private String ip;

    public MesDefectAlarmConfigJob() {
        super(MesDefectAlarmConfigJob.class, "缺陷告警配置job按工厂划分");
        this.setMultiInstance(true);
    }

    @Override
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        try {
            String jobParam = this.getJobParam();

            if (StringUtils.isBlank(jobParam)) {
                throw new IllegalArgumentException("jobc参数为空，请检查参数");
            }

            Map<String, Object> paramMap = new HashMap<>();

            try {
                paramMap = (Map<String, Object>) JSONObject.parse(jobParam);
            } catch (Exception e) {
                LOGGER.error("参数格式不是JSON");
                return;

            }

            if (CollectionUtils.isEmpty(paramMap)) {
                LOGGER.error("检测数据同步定时任务,没有配置参数");
                return;
            }

            List<String> organizeCodeList = (List<String>) paramMap.get("org");

            for (String organizeCode : organizeCodeList) {

                defectAlarmConfigService.saveDefectAlarmConfigByJob(organizeCode);
            }
            LOGGER.info("缺陷告警配置job结束 ----- end");

        } catch (Exception e) {
            LOGGER.error("SAP接口表数据同步作业任务结束e:{}", e.toString());
            //sendErrorMessage(e.toString());
        }
    }

}
