package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IBaseMesService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.jpa.dao.BaseRepository;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import org.springframework.beans.factory.annotation.Autowired;

import java.lang.reflect.ParameterizedType;
import java.util.List;

/**
 * @Description:
 * @CreateDate: 2022/12/1 12:15
 * @Author: simon.song
 */
public abstract class BaseMesService<T extends BaseBean> implements IBaseMesService<T> {
    @Autowired
    protected BaseRepository<T, Long> baseRDao;

    /**
     * 泛型class
     */
    protected Class<T> mesClass;

    /**
     * 初始化
     */
    public BaseMesService() {
        // 通过反射获取T的真实类型
        ParameterizedType pt = (ParameterizedType) this.getClass().getGenericSuperclass();
        this.mesClass = (Class<T>) pt.getActualTypeArguments()[0];
    }

    @Override
    public Class<T> getMesClass() {
        return mesClass;
    }

    @Override
    public ListPager<T> queryPager(T bean, Pager pager) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode(), bean.getIsValid());
        setPackQueryBean(bean, packBean);
        pager = PagerHelper.getPager(pager, baseRDao.findByHqlWhereCount(packBean));
        List<T> list = baseRDao.findByHqlWherePage(packBean, pager);
        setBeanModel(list);
        return new ListPager<>(list, pager);
    }

    @Override
    public ListPager<T> queryPagerWhere(Pager pager, DdlPackBean packBean) {
        pager = PagerHelper.getPager(pager, baseRDao.findByHqlWhereCount(packBean));
        List<T> list = baseRDao.findByHqlWherePage(packBean, pager);
        setBeanModel(list);
        return new ListPager<>(list, pager);
    }


    @Override
    public List<T> findAll(DdlPackBean packBean) {
        setPackQueryBean(packBean);
        return baseRDao.findByHqlWhere(packBean);
    }

    @Override
    public T get(Long id) {
        return baseRDao.getById(id);
    }

    @Override
    public T insert(T bean) {
        onInsertBean(bean);
        bean = baseRDao.insert(bean);
        return bean;
    }

    @Override
    public List<T> insertBatch(List<T> beanList) {
        for (T bean : beanList) {
            onInsertBean(bean);
            insert(bean);
        }
        return beanList;
    }

    @Override
    public T update(T bean) {
        onUpdateBean(bean);
        T originBean = baseRDao.getById(bean.getId());
        onModifyBean();
        ConvertBean.serviceModelUpdate(bean, AuthUtil.getSessionUser().getUserName());
        baseRDao.update(bean);
        return originBean;
    }

    @Override
    public void updates(Iterable<T> beans) {
        for (T bean : beans) {
            onUpdateBean(bean);
            ConvertBean.serviceModelUpdate(bean, AuthUtil.getSessionUser().getUserName());
            baseRDao.update(bean);
        }
    }

    @Override
    public void deleteWeaklyById(Long id, String userName) {
        onDeleteId(id);
        baseRDao.deleteWeaklyById(id, userName);
    }

    @Override
    public void deleteWeaklyByIds(Long[] ids, String userName) {
        for (Long id : ids) {
            onDeleteId(id);
            baseRDao.deleteWeaklyById(id, userName);
        }
    }

    @Override
    public void deleteById(Long id) {
        onDeleteId(id);
        baseRDao.deleteById(id);
    }

    @Override
    public void deleteByIds(Long[] ids) {
        for (Long id : ids) {
            onDeleteId(id);
            baseRDao.deleteById(id);
        }
    }

    @Override
    public int updateValid(Long id, int status, String userName) {
        if (status != CommonEnumUtil.IS_VAILD.INVAILD.getValue() && status != CommonEnumUtil.IS_VAILD.VAILD.getValue()) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("启用，禁用枚举状态值【%s】未被定义", status)
                    .build();
        }
        return baseRDao.updateByProperties(new String[]{"id"}, new Object[]{id},
                new String[]{MesConstWords.IS_VALID, "modifyUser", "modifyDatetime"},
                new Object[]{status, userName, TimeTool.getNowTime(true)});
    }

    protected void onInsertBean(T item) {
    }

    protected void onUpdateBean(T item) {
    }

    protected void onModifyBean() {
    }

    protected void setBeanModel(List<T> list) {
    }

    protected void setPackQueryBean(T bean, DdlPackBean packBean) {
    }

    protected void setPackQueryBean(DdlPackBean packBean) {
    }

    protected void onDeleteId(Long id) {
    }
}
