package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCarSeriesLoosePartService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCarSeriesLoosePart;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class MesCarSeriesLoosePartService extends BaseMesService<MesCarSeriesLoosePart> implements IMesCarSeriesLoosePartService {

    protected void onInsertBean(MesCarSeriesLoosePart item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getPartNo(), "零件号不能为空");
        ValidatorBean.checkNotNull(item.getPartName(), "零件名称不能为空");
        ValidatorBean.checkNotNull(item.getLoosePartType(), "散件类型不能为空");

        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", partPackBean);
        boolean flg = baseRDao.isExitByHql(partPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件号已经存在，请检查数据", item.getPartNo())
                    .build();
        }


    }

    protected void onUpdateBean(MesCarSeriesLoosePart item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getPartNo(), "零件号不能为空");
        ValidatorBean.checkNotNull(item.getPartName(), "零件名称不能为空");
        ValidatorBean.checkNotNull(item.getLoosePartType(), "散件类型不能为空");

        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", partPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", partPackBean);
        boolean flg = baseRDao.isExitByHql(partPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件号已经存在，请检查数据", item.getPartNo())
                    .build();
        }

    }

}
