package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCustomerRoadService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerRoad;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class MesCustomerRoadService extends BaseMesService<MesCustomerRoad> implements IMesCustomerRoadService {


    protected void onInsertBean(MesCustomerRoad item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCustomerCode(), "客户代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCustomerCode(), "customerCode", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getRoadCode(), "roadCode", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】客户代码【%s】道口代码已经存在，请检查数据", item.getCustomerCode(), item.getRoadCode())
                    .build();
        }

    }

    protected void onUpdateBean(MesCustomerRoad item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCustomerCode(), "客户代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCustomerCode(), "customerCode", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getRoadCode(), "roadCode", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】客户代码【%s】道口代码已经存在，请检查数据", item.getCustomerCode(), item.getRoadCode())
                    .build();
        }
    }

    protected void setPackQueryBean(MesCustomerRoad bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getCustomerCode(), "customerCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getRoadCode(), "roadCode", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getType(), "type", packBean);
    }

}
