package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.api.base.IMesDefectAlarmConfigService;
import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesDefectRecordDao;
import cn.estsh.i3plus.ext.mes.pojo.model.MesDefectRecordModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesDefect;
import cn.estsh.i3plus.pojo.mes.bean.MesDefectAlarmConfig;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.repository.MesDefectRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartRepository;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


@Service
@Slf4j
public class MesDefectAlarmConfigService extends BaseMesService<MesDefectAlarmConfig> implements IMesDefectAlarmConfigService {


    @Autowired
    private IMesDefectRecordDao defectRecordDao;
    @Autowired
    private MesPartRepository partRepository;
    @Autowired
    private MesDefectRepository defectRepository;
    @Autowired
    private IMesConfigService configService;


    protected void onInsertBean(MesDefectAlarmConfig item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getPartTypeCode(), "零件类型不能为空");

    }

    protected void onUpdateBean(MesDefectAlarmConfig item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getPartTypeCode(), "零件类型不能为空");
    }

    protected void setPackQueryBean(MesDefectAlarmConfig bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getDefectCode(), "defectCode", packBean);
//        DdlPreparedPack.getNumEqualPack(bean.getTotalTriggerCount(), "totalTriggerCount", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getDefectLocationCode(), "defectLocationCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getPartTypeCode(), "partTypeCode", packBean);
    }

    @Override
    public void saveDefectAlarmConfigByJob(String org) {

        String count = configService.getCfgValue(org, "DEFECT_ALARM_CONFIG_COUNT");
        String createDateTimeStart = TimeTool.getToday() + MesExtConstWords.APPEND_ZERO_HMS;
        String createDateTimeEnd = TimeTool.getToday() + MesExtConstWords.APPEND_24_HMS;
        //不良类型
        List<MesDefectRecordModel> modelList = defectRecordDao.queryMesDefectRecordGroupBy(org, Integer.parseInt(count), createDateTimeStart, createDateTimeEnd);

        for (MesDefectRecordModel model : modelList) {
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
            DdlPreparedPack.getStringEqualPack(model.getDefectCode(), "defectCode", packBean);
            DdlPreparedPack.getStringLikerPack(model.getDefectLocationCode(), "defectLocationCode", packBean);
            DdlPreparedPack.getStringEqualPack(model.getPartTypeCode(), "partTypeCode", packBean);
            DdlPreparedPack.getStringEqualPack(model.getPartNo(), "partNo", packBean);
            DdlPreparedPack.getNumEqualPack(model.getSides(), "sides", packBean);
            boolean flg = baseRDao.isExitByHql(packBean);
            if (!flg) {
                DdlPackBean defectPackBean = DdlPackBean.getDdlPackBean(org);
                DdlPreparedPack.getStringEqualPack(model.getDefectCode(), "defectCode", defectPackBean);
                MesDefect defect = defectRepository.getByProperty(defectPackBean);

                DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(org);
                DdlPreparedPack.getStringEqualPack(model.getPartNo(), "partNo", partPackBean);
                MesPart part = partRepository.getByProperty(partPackBean);

                MesDefectAlarmConfig config = new MesDefectAlarmConfig();
                config.setPartNo(part.getPartNo());
                config.setPartName(part.getPartName());
                config.setDefectCode(defect.getDefectCode());
                config.setDefectName(defect.getDefectName());
                config.setDefectLocationCode(model.getDefectLocationCode());
                config.setPartTypeCode(part.getPartTypeCode());
                config.setPartTypeName(part.getPartTypeName());
                config.setSides(model.getSides());
                config.setOrganizeCode(org);

                ConvertBean.serviceModelInitialize(config, "xxljob");
                baseRDao.save(config);
            }
        }


    }
}
