package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesProcessCraftCfgService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesProcessCraftCfg;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @Description : 工序工艺关系
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/11 13:55
 * @Modify:
 **/
@Service
@Slf4j
public class MesProcessCraftCfgServiceImpl extends BaseMesService<MesProcessCraftCfg> implements IMesProcessCraftCfgService {

    protected void setPackQueryBean(MesProcessCraftCfg bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getCraftCode(), "craftCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getProcessCode(), "processCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCraftName(), "craftName", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getProcessName(), "processName", packBean);
    }

    protected void onInsertBean(MesProcessCraftCfg item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCraftCode(), "工艺代码不能为空");
        ValidatorBean.checkNotNull(item.getProcessCode(), "工序代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getProcessCode(), "processCode", roadPackBean);

        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("工序代码【%s】已经存在，请检查数据", item.getCraftCode())
                    .build();
        }
    }

    protected void onUpdateBean(MesProcessCraftCfg item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCraftCode(), "工艺代码不能为空");
        ValidatorBean.checkNotNull(item.getProcessCode(), "工序代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getProcessCode(), "processCode", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("工序代码【%s】已经存在，请检查数据", item.getCraftCode())
                    .build();
        }
    }
}
