package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesProdRuleSortCfgService;
import cn.estsh.i3plus.ext.mes.api.base.IMesTimeEfficientCfgService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesProdRuleSortCfg;
import cn.estsh.i3plus.pojo.mes.bean.MesProdRuleSortCfg;
import cn.estsh.i3plus.pojo.mes.bean.MesTimeEfficientCfg;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

@Service
@Slf4j
public class MesProdRuleSortCfgService extends BaseMesService<MesProdRuleSortCfg> implements IMesProdRuleSortCfgService {

    protected void setPackQueryBean(MesProdRuleSortCfg bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartName(), "partName", packBean);

    }

    @Override
    public MesProdRuleSortCfg insert(MesProdRuleSortCfg bean) {
        //插入前构造并校验数据
        onInsertBean(bean);

        bean = baseRDao.insert(bean);
        return bean;
    }

    protected void onInsertBean(MesProdRuleSortCfg item) {
        // 数据校验
        //ValidatorBean.checkNotNull(item.getInPartNo(), "进件零件号不能为空");
        ValidatorBean.checkNotNull(item.getWorkCenterCode(), "产线不能为空");
        ValidatorBean.checkNotNull(item.getWorkCellCode(), "工位不能为空");
        ValidatorBean.checkNotNull(item.getPartNo(), "零件号不能为空");

        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWorkCellCode(), "workCellCode", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", seriesPackBean);


        boolean flg = baseRDao.isExitByHql(seriesPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("该排序加工规则已存在，请检查数据")
                    .build();
        }

    }

    protected void onUpdateBean(MesProdRuleSortCfg item) {
        // 数据校验
        //ValidatorBean.checkNotNull(item.getInPartNo(), "进件零件号不能为空");
        ValidatorBean.checkNotNull(item.getWorkCenterCode(), "产线不能为空");
        ValidatorBean.checkNotNull(item.getWorkCellCode(), "工位不能为空");
        ValidatorBean.checkNotNull(item.getPartNo(), "零件号不能为空");

        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWorkCellCode(), "workCellCode", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", seriesPackBean);


        boolean flg = baseRDao.isExitByHql(seriesPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("该排序加工规则已存在，请检查数据")
                    .build();
        }
    }
}
