package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.api.base.IMesShippingOrderManagementService;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.MesJisShippingRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartShippingGroupRepository;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class MesShippingOrderManagementService extends BaseMesService<MesShippingOrderManagement> implements IMesShippingOrderManagementService {

    @Autowired
    private MesShippingOrderManagementDetailService detailService;

    @Autowired
    private MesPartShippingGroupDetailService shippingGroupDetailService;

    @Autowired
    private MesPartShippingGroupRepository partShippingGroupRepository;

    @Autowired
    private MesJisShippingRepository jisShippingRepository;
    @Autowired
    private IMesConfigService configService;

    @Autowired
    private MesPartRepository partRepository;


    protected void setPackQueryBean(MesShippingOrderManagement bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getShippingGroupNo(), "shippingGroupNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustCode(), "custCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustPlantCode(), "custPlantCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getShippingGroupNo(), "shippingGroupNo", packBean);
        if (!StringUtils.isEmpty(bean.getReleaseTimeStart()) || !StringUtils.isEmpty(bean.getReleaseTimeEnd())) {
            DdlPreparedPack.timeBuilder(bean.getReleaseTimeStart(), bean.getReleaseTimeEnd(), "releaseTime", packBean, false);
        }
    }

    @Override
    public MesShippingOrderManagement insert(MesShippingOrderManagement bean) {
        //插入前构造并校验数据
        onInsertBean(bean);

        DdlPackBean partShipGroupPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(bean.getShippingGroupNo(), "shippingGroupNo", partShipGroupPackBean);
        MesPartShippingGroup partShippingGroup = partShippingGroupRepository.getByProperty(partShipGroupPackBean);

        if (StringUtils.isEmpty(partShippingGroup)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组数据为空，请检查数据", bean.getShippingGroupNo())
                    .build();
        }

        if (StringUtils.isEmpty(partShippingGroup.getIsLoading()) || partShippingGroup.getIsLoading() == 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否有装车不能为空，请检查数据", bean.getShippingGroupNo())
                    .build();
        }
        if (StringUtils.isEmpty(partShippingGroup.getIsFirstInstall()) || partShippingGroup.getIsFirstInstall() == 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否先装车不能为空，请检查数据", bean.getShippingGroupNo())
                    .build();
        }
        if (StringUtils.isEmpty(partShippingGroup.getIsEnableMemory()) || partShippingGroup.getIsEnableMemory() == 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否启用记忆不能为空，请检查数据", bean.getShippingGroupNo())
                    .build();
        }

        bean.setIsLoading(partShippingGroup.getIsLoading());
        bean.setIsFirstInstall(partShippingGroup.getIsFirstInstall());
        bean.setIsEnableMemory(partShippingGroup.getIsEnableMemory());

        //查询物料发运组明细数据
        DdlPackBean shipGroupPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(bean.getShippingGroupNo(), "shippingGroupNo", shipGroupPackBean);
        List<MesPartShippingGroupDetail> groupDetailList = shippingGroupDetailService.findAll(shipGroupPackBean);
        if (CollectionUtils.isEmpty(groupDetailList)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组明细数据为空，请检查数据", bean.getShippingGroupNo())
                    .build();
        }

        //发运单明细
        List<MesShippingOrderManagementDetail> detailInsertList = new ArrayList<>();

        MesShippingOrderManagement finalBean = bean;
        groupDetailList.forEach(k -> {
            MesShippingOrderManagementDetail detail = new MesShippingOrderManagementDetail();

            detail.setShippingOrderNo(finalBean.getShippingOrderNo());
            detail.setOrganizeCode(k.getOrganizeCode());
            detail.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue());
            ConvertBean.serviceModelInitialize(detail, AuthUtil.getSessionUser().getUserName());
            BeanUtils.copyProperties(k, detail, "id");
            detailInsertList.add(detail);
        });

        detailService.insertBatch(detailInsertList);
        bean = baseRDao.insert(bean);
        return bean;
    }


    @Override
    public MesShippingOrderManagement update(MesShippingOrderManagement bean) {
        //更新前构造并校验数据
        onUpdateBean(bean);
        MesShippingOrderManagement originBean = baseRDao.getById(bean.getId());
        if (StringUtils.isEmpty(originBean)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("请选择需要操作的资源。")
                    .build();
        }

        //判断是否修改发运组，如果不修改发运组，则不修改明细
        if (!bean.getShippingGroupNo().equals(originBean.getShippingGroupNo())) {

            DdlPackBean detailPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getShippingOrderNo(), "shippingOrderNo", detailPackBean);
            List<MesShippingOrderManagementDetail> detailDeleteList = detailService.findAll(detailPackBean);
            detailDeleteList.forEach(k -> {
                k.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
            });
            //查询物料发运组数据
            DdlPackBean shipGroupPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getShippingGroupNo(), "shippingGroupNo", shipGroupPackBean);
            List<MesPartShippingGroupDetail> shippingGroupDetailList = shippingGroupDetailService.findAll(shipGroupPackBean);
            if (CollectionUtils.isEmpty(shippingGroupDetailList)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("【%s】发运组明细数据为空，请检查数据", bean.getShippingGroupNo())
                        .build();
            }
            //发运单明细
            List<MesShippingOrderManagementDetail> detailInsertList = new ArrayList<>();
            shippingGroupDetailList.forEach(k -> {
                MesShippingOrderManagementDetail detail = new MesShippingOrderManagementDetail();

                detail.setOrganizeCode(k.getOrganizeCode());
//                ConvertBean.serviceModelInitialize(detail, "LML");
                ConvertBean.serviceModelInitialize(detail, AuthUtil.getSessionUser().getUserName());
                BeanUtils.copyProperties(k, detail, "id");
                detailInsertList.add(detail);
            });
            //软删原明细数据
            detailService.updates(detailDeleteList);
            //新增新明细数据
            detailService.insertBatch(detailInsertList);
        }

        onModifyBean();
//        ConvertBean.serviceModelUpdate(bean, "LML");
        ConvertBean.serviceModelUpdate(bean, AuthUtil.getSessionUser().getUserName());
        baseRDao.update(bean);
        return originBean;
    }


    @Override
    public void deleteWeaklyByIds(Long[] ids, String userName) {

        for (Long id : ids) {
            MesShippingOrderManagement bean = baseRDao.getById(id);
            if (StringUtils.isEmpty(bean)) continue;
            DdlPackBean detailPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getShippingOrderNo(), "shippingGroupNo", detailPackBean);
            List<MesShippingOrderManagementDetail> detailDeleteList = detailService.findAll(detailPackBean);
            detailDeleteList.forEach(k -> {
                k.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
            });
            bean.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            ConvertBean.serviceModelUpdate(bean, AuthUtil.getSessionUser().getUserName());
            baseRDao.update(bean);
            detailService.updates(detailDeleteList);
        }
    }


    protected void onInsertBean(MesShippingOrderManagement item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getShippingOrderNo(), "发运单不能为空");
        ValidatorBean.checkNotNull(item.getSeq(), "顺序号不能为空");
        ValidatorBean.checkNotNull(item.getCustCode(), "客户代码不能为空");
        ValidatorBean.checkNotNull(item.getCustPlantCode(), "客户工厂代码不能为空");
        ValidatorBean.checkNotNull(item.getShippingGroupNo(), "发运组代码不能为空");
        ValidatorBean.checkNotNull(item.getStatus(), "状态不能为空");

        DdlPackBean shipGroupPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getShippingOrderNo(), "shippingOrderNo", shipGroupPackBean);
        boolean flg = baseRDao.isExitByHql(shipGroupPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】发运单已经存在，请检查数据", item.getShippingOrderNo())
                    .build();
        }


    }

    protected void onUpdateBean(MesShippingOrderManagement item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getShippingOrderNo(), "发运单不能为空");
        ValidatorBean.checkNotNull(item.getSeq(), "顺序号不能为空");
        ValidatorBean.checkNotNull(item.getCustCode(), "客户代码不能为空");
        ValidatorBean.checkNotNull(item.getCustPlantCode(), "客户工厂代码不能为空");
        ValidatorBean.checkNotNull(item.getShippingGroupNo(), "发运组代码不能为空");
        ValidatorBean.checkNotNull(item.getStatus(), "状态不能为空");
    }


    @Override
    public void saveShipping(MesShippingOrderManagement bean) {

        MesShippingOrderManagement originBean = baseRDao.getById(bean.getId());

        DdlPackBean detailPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(originBean.getShippingOrderNo(), "shippingOrderNo", detailPackBean);
        List<MesShippingOrderManagementDetail> detailDeleteList = detailService.findAll(detailPackBean);

        ;
        List<MesJisShipping> jisShippingList = new ArrayList<>();
        detailDeleteList.forEach(k -> {

            DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(k.getPartNo(), "partNo", packBean);
            MesPart part = partRepository.getByProperty(packBean);

            MesJisShipping jisShipping = new MesJisShipping();
            BeanUtils.copyProperties(originBean, jisShipping, "id");
            BeanUtils.copyProperties(k, jisShipping, "id");
            jisShipping.setFactoryCode(originBean.getOrganizeCode());
            jisShipping.setCusCode(originBean.getCustCode());
            jisShipping.setShippingTime(TimeTool.getNowTime(true));
            jisShipping.setDock(originBean.getSendDockCode());
            jisShipping.setErpWarerhouse(configService.getCfgValue(bean.getOrganizeCode(), "ERP_WARERHOUSE"));
            jisShipping.setErpWarehous(configService.getCfgValue(bean.getOrganizeCode(), "ERP_WAREHOUS"));
            jisShipping.setCusDock(originBean.getCustDockCode());
            jisShipping.setCustOrderNo(k.getCustOrderNo());
            jisShipping.setSerialNumber(k.getBarcode());
            jisShipping.setQuantityPerCar(detailDeleteList.size());
            jisShipping.setShippingOrderCode(originBean.getShippingOrderNo());
            jisShipping.setVin(k.getVin());
            jisShipping.setCarNo(k.getVin());
            if(!StringUtils.isEmpty(part)){
                jisShipping.setUnit(part.getUnit());
            }
            jisShipping.setOffTime(TimeTool.getNowTime(true));
            ConvertBean.serviceModelInitialize(jisShipping, AuthUtil.getSessionUser().getUserName());
            jisShippingList.add(jisShipping);
        });

        //修改当前发运单状态
        originBean.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue());

        //修改当前发运单明细状态
        detailDeleteList.forEach(k -> {
            ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
            k.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue());
        });

        ConvertBean.serviceModelUpdate(bean, AuthUtil.getSessionUser().getUserName());
        baseRDao.save(originBean);
        detailService.updates(detailDeleteList);

        jisShippingRepository.saveAll(jisShippingList);

    }
}
