package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesWorkCellBinService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesShippingOrderManagement;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCellBin;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class MesWorkCellBinService extends BaseMesService<MesWorkCellBin> implements IMesWorkCellBinService {

    protected void onInsertBean(MesWorkCellBin item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getWorkCenterCode(), "产线不能为空");
        ValidatorBean.checkNotNull(item.getWorkCellCode(), "工位不能为空");

        DdlPackBean cellBinPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWorkCellCode(), "workCellCode", cellBinPackBean);
        boolean flg = baseRDao.isExitByHql(cellBinPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】工位已经存在，请检查数据", item.getWorkCenterCode())
                    .build();
        }

        cellBinPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getLocateNo(), "locateNo", cellBinPackBean);
        boolean flg1 = baseRDao.isExitByHql(cellBinPackBean);
        if (flg1) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】bin位已经存在，请检查数据", item.getLocateNo())
                    .build();
        }
    }

    protected void onUpdateBean(MesWorkCellBin item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getWorkCenterCode(), "产线不能为空");
        ValidatorBean.checkNotNull(item.getWorkCellCode(), "工位不能为空");

        DdlPackBean cellBinPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWorkCellCode(), "workCellCode", cellBinPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", cellBinPackBean);
        boolean flg = baseRDao.isExitByHql(cellBinPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】工位已经存在，请检查数据", item.getWorkCenterCode())
                    .build();
        }

        cellBinPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getLocateNo(), "locateNo", cellBinPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", cellBinPackBean);
        boolean flg1 = baseRDao.isExitByHql(cellBinPackBean);
        if (flg1) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】bin位已经存在，请检查数据", item.getLocateNo())
                    .build();
        }
    }

}
