package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPartService;
import cn.estsh.i3plus.ext.mes.api.base.IMesProductOffLineService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderToSapService;
import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesWorkOrderDao;
import cn.estsh.i3plus.ext.mes.apiservice.unit.exception.MesException;
import cn.estsh.i3plus.mes.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.mes.apiservice.util.DateUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.MesBomRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProductVersionRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesShiftRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesWorkCenterRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Service
@Slf4j
public class MesWorkOrderService extends BaseMesService<MesWorkOrder> implements IMesWorkOrderService {
    @Autowired
    private MesWorkOrderLogService workOrderLogService;
    @Autowired
    private ISyncFuncService syncFuncService;
    @Autowired
    private IMesWorkOrderToSapService iMesWorkOrderToSapService;
    @Autowired
    private IMesPartService iMesPartService;
    @Autowired
    private MesProductVersionRepository mesProductVersionRDao;
    @Autowired
    private MesBomRepository mesBomRDao;
    @Autowired
    private IMesProductOffLineService mesProductOffLineService;
    @Autowired
    private IMesWorkOrderDao mesWorkOrderDao;
    @Autowired
    private MesWorkCenterRepository mesWorkCenterRDao;
    @Autowired
    private MesShiftRepository mesShiftRDao;

    @Override
    public MesWorkOrder insert(MesWorkOrder bean) {
        //插入前构造数据
        onInsertBean(bean);

        //获取相同类型 相同标识 生产序号最大值的
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderStatus(), "workOrderStatus", orderPackBean);
        DdlPreparedPack.getNumEqualPack(bean.getOrderFlag(), "orderFlag", orderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"produceSeq"}, orderPackBean);
        MesWorkOrder result = baseRDao.getByProperty(orderPackBean);


        GenSerialNoModel genSerialNoModel = new GenSerialNoModel();
        //若工单类型为排序 则根据排序插入工单数量插入多行 工单数量为1
        if (bean.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            Integer produceSeq = 0;
            if (null != result && result.getProduceSeq() != null) {
                produceSeq = result.getProduceSeq();
            }

            List<MesWorkOrderLog> orderLogList = new ArrayList<>();
            //生成工单号
            genSerialNoModel = new GenSerialNoModel("ORDER_NO_JIS_SORT");
            genSerialNoModel.setPartNo(bean.getPartNo());
            List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();

            MesWorkOrder copyMesWorkOrder = new MesWorkOrder();
            BeanUtils.copyProperties(bean, copyMesWorkOrder);
            copyMesWorkOrder.setProduceSeq(produceSeq + 1);
            copyMesWorkOrder.setQty(1D);
            copyMesWorkOrder.setOrderNo(resultList.get(0) + "_" + copyMesWorkOrder.getOrganizeCode());
            ConvertBean.serviceModelInitialize(copyMesWorkOrder, bean.getCreateUser());

            MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
            BeanUtils.copyProperties(copyMesWorkOrder, workOrderLog, "id");
            orderLogList.add(workOrderLog);

            baseRDao.insert(copyMesWorkOrder);
            workOrderLogService.insertBatch(orderLogList);
            //写入接口表
            //iMesWorkOrderToSapService.insert(bean);
            return bean;
        } else if (bean.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()) {
            //生成工单号
            genSerialNoModel = new GenSerialNoModel("ORDER_NO_JIS_UN_SORT");
        } else {
            //生成工单号
            genSerialNoModel = new GenSerialNoModel("ORDER_NO_BTO");
            Integer produceSeq = 0;
            if (null != result && result.getProduceSeq() != null) {
                produceSeq = result.getProduceSeq();
            }
            bean.setProduceSeq(produceSeq + 1);
            GenSerialNoModel orderSeqSerialNoModel = new GenSerialNoModel("ORDER_NO_BTO_SEQ");
            orderSeqSerialNoModel.setPartNo(bean.getPartNo());
            List<String> resultList = syncFuncService.syncSerialNo(orderSeqSerialNoModel, bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();
            bean.setOrderSeq(Integer.parseInt(resultList.get(0)));
        }
        genSerialNoModel.setPartNo(bean.getPartNo());
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();
        bean.setOrderNo(resultList.get(0) + "_" + bean.getOrganizeCode());
        bean.setQty(bean.getQty());

        ConvertBean.serviceModelInitialize(bean, bean.getCreateUser());

        MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
        BeanUtils.copyProperties(bean, workOrderLog, "id");
        workOrderLogService.insert(workOrderLog);
        bean = baseRDao.insert(bean);
        //写入接口表
        //iMesWorkOrderToSapService.insert(bean);
        return bean;
    }

    @Override
    public MesWorkOrder update(MesWorkOrder bean) {
        //回传sap
        boolean sendToSap = false;

        MesWorkOrder result = baseRDao.getById(bean.getId());
        //若工单类型为排序 则根据排序插入工单数量插入多行 工单数量为1
        if (result.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            if (!Objects.equals(bean.getQty(), result.getQty())) {
                MesException.throwMesBusiException(String.format("工单类型为【%s】 不需要修改数量",
                        MesExtEnumUtil.ORDER_TYPE.SORT.getDescription()));
            }
        } else {
            //工单数量只能比原来的大
            if (bean.getQty() < result.getQty()) {
                MesException.throwMesBusiException(String.format("工单数量不能小于工单源数量【%s】",
                        result.getQty()));
            }
            //修改状态或者指令下发需要传给SAP
            if (!Objects.equals(bean.getQty(), result.getQty())) {
                sendToSap = true;
            }
        }
        if (MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue() == bean.getWorkOrderStatus() ||
                MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue() == bean.getWorkOrderStatus()) {
            sendToSap = true;
        }
        //在加工的工单不能修改物料信息
        if (result.getWorkOrderStatus() >= MesExtEnumUtil.ORDER_STATUS.process.getValue()) {
            if (!Objects.equals(bean.getPartNo(), result.getPartNo())) {
                MesException.throwMesBusiException(String.format("工单状态为【%s】不允许修改物料",
                        MesExtEnumUtil.ORDER_TYPE.valueOfDescription(result.getWorkOrderStatus())));
            }
        }

        result.setQty(bean.getQty());
        //更新已汇报
        if (bean.getReportedQty() >= 0) {
            result.setReportedQty(bean.getReportedQty());
        }
        if (bean.getUnCompleteQty() >= 0) {
            result.setUnCompleteQty(bean.getUnCompleteQty());
        }
        if (bean.getAdjustQty() > 0) {
            result.setAdjustQty(bean.getAdjustQty());
        }
        //更新状态
        if (bean.getWorkOrderStatus() > 0) {
            if (MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue() == bean.getWorkOrderStatus()) {
                if (MesExtEnumUtil.ORDER_STATUS.CREATE.getValue() != result.getWorkOrderStatus()) {
                    MesException.throwMesBusiException(String.format("工单状态为【%s】不为创建，不允许发布",
                            MesExtEnumUtil.ORDER_TYPE.valueOfDescription(result.getWorkOrderStatus())));
                }
            }
            result.setWorkOrderStatus(bean.getWorkOrderStatus());
        }
        ConvertBean.serviceModelInitialize(bean, AuthUtil.getSessionUser().getUserName());

        MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
        BeanUtils.copyProperties(result, workOrderLog, "id");
        workOrderLogService.insert(workOrderLog);
        //关闭工单
        if (MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue() == bean.getWorkOrderStatus()) {
            bean.setQty(0d);
            sendToSap = true;
        }
        //发送给SAP
        if (sendToSap) {
            //iMesWorkOrderToSapService.insert(bean);
        }

        baseRDao.update(result);
        return bean;
    }


    protected void setPackQueryBean(MesWorkOrder bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getOrderNo(), "orderNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNameRdd(), "partNameRdd", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCreateUser(), "createUser", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustomerPartNo(), "customerPartNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderStatus(), "workOrderStatus", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderType(), "workOrderType", packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getCreateDateTimeStart(), "createDatetime", packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getCreateDateTimeEnd(), "createDatetime", packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getStartTimeStart(), "planStartTime", packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getStartTimeEnd(), "planStartTime", packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getEndTimeStart(), "planEndTime", packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getEndTimeEnd(), "planEndTime", packBean);
        packBean.setOrderByStr(" order by createDatetime desc ");
    }

    protected void onInsertBean(MesWorkOrder item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getWorkOrderType(), "工单类型不能为空");
        ValidatorBean.checkNotNull(item.getWorkCenterCode(), "产线不能为空");

        //ValidatorBean.checkNotNull(item.getShiftCode(), "班次不能为空");

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        if (!mesWorkCenterRDao.isExitByHql(ddlPackBean)) {
            MesException.throwMesBusiException("产线【%s】不存在", item.getWorkCenterCode());
        }
        //校验标识
        //若工单类型为排序
        if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            ValidatorBean.checkNotNull(item.getOrderFlag(), "标识不能为空");
            ValidatorBean.checkNotNull(item.getPlanStartTime(), "计划开始时间不能为空");
            ValidatorBean.checkNotNull(item.getPlanEndTime(), "计划结束时间不能为空");
            //若工单类型为排序 标识为 N E P S
            if (item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.S.getValue() &&
                    item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue() &&
                    item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue() &&
                    item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue()) {
                throw ImppExceptionBuilder.newInstance()
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail(String.format("工单标识为【%s】请选择 【%s】、【%s】、【%s】、【%s】 标识",
                                item.getOrderFlag(), MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.S.getDescription()
                                , MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getDescription(),
                                MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getDescription(),
                                MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getDescription()))
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .build();
            }
            //校验vin号是否重复
            ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(item.getVinCode(), "vinCode", ddlPackBean);
            if (baseRDao.isExitByHql(ddlPackBean)) {
                MesException.throwMesBusiException("vin号【%s】重复", item.getVinCode());
            }

        } else if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()) {
            ValidatorBean.checkNotNull(item.getOrderFlag(), "标识不能为空");
            ValidatorBean.checkNotNull(item.getProductVersion(), "生产版本不能为空");
            if (item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue() &&
                    item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue()) {
                throw ImppExceptionBuilder.newInstance()
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail(String.format("工单标识为【%s】请选择 【%s】、【%s】 标识",
                                item.getOrderFlag(), MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getDescription(),
                                MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getDescription()))
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .build();
            }
            if (StringUtil.isEmpty(item.getPlanStartTime()) || StringUtil.isEmpty(item.getPlanEndTime())) {
                ValidatorBean.checkNotNull(item.getPlanDate(), "计划日期不能为空");
                ValidatorBean.checkNotNull(item.getShiftCode(), "班次不能为空");
                //查询班次信息
                ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", ddlPackBean);
                DdlPreparedPack.getStringEqualPack(item.getShiftCode(), "shiftCode", ddlPackBean);
                MesShift mesShift = mesShiftRDao.getByProperty(ddlPackBean);
                if (Objects.isNull(mesShift)) {
                    MesException.throwMesBusiException("产线【%s】班次【%s】信息不存在", item.getWorkCenterCode(), item.getShiftCode());
                }
                item.setShiftName(mesShift.getShiftName());
                //拼接
                item.setPlanStartTime(item.getPlanDate() + " " + mesShift.getStartTime());
                item.setPlanEndTime(item.getPlanDate() + " " + mesShift.getEndTime());
                try {
                    int windowStartEndTimeCompare = this.compareDateTime(DateUtil.BASE_FORMAT, item.getPlanStartTime(), item.getPlanEndTime());
                    if (windowStartEndTimeCompare > 0) {
                        item.setPlanEndTime(TimeTool.pareDateToString(DateUtil.addDays(item.getPlanEndTime(),1)));
                    }
                } catch (Exception e) {
                    MesException.throwMesBusiException("时间格式异常");
                }
            }
            //校验物料生产版本是否存在
            ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(item.getProductVersion(), "productVersion", ddlPackBean);
            if (!mesProductVersionRDao.isExitByHql(ddlPackBean)) {
                MesException.throwMesBusiException("物料【%s】产线【%s】生产版本【%s】信息不存在", item.getPartNo(),item.getWorkCenterCode(), item.getProductVersion());
            }

        } else {
            ValidatorBean.checkNotNull(item.getPlanStartTime(), "计划开始时间不能为空");
            ValidatorBean.checkNotNull(item.getPlanEndTime(), "计划结束时间不能为空");
            //拼接
//            item.setPlanStartTime(item.getPlanStartTime() + " " + "00:00:00");
//            item.setPlanEndTime(item.getPlanEndTime() + " " + "23:59:59");
        }
        //物料名称不存在，从物料信息中获取
        if (StringUtil.isEmpty(item.getPartNameRdd())) {
            MesPart mesPart = iMesPartService.getPartByPartNo(item.getPartNo(), item.getOrganizeCode());
            item.setPartNameRdd(mesPart.getPartName());
        }
        item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.CREATE.getValue());
    }

    protected void onUpdateBean(MesWorkOrder item) {

    }

    public int compareDateTime(String formatStr, String startTimeStr, String endTimeStr) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat(formatStr);
        Date startTime = sdf.parse(startTimeStr);
        Date endTime = sdf.parse(endTimeStr);
        return startTime.compareTo(endTime);
    }

    @Override
    public void doProductReport(MesWorkOrder mesWorkOrder) {
        //查询物料信息
        MesPart mesPart = iMesPartService.getPartByPartNo(mesWorkOrder.getPartNo(), mesWorkOrder.getOrganizeCode());
        //获取生产版本
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (null == mesProductVersion) {
            MesException.throwMesBusiException("物料【%s】生产版本信息不存在", mesWorkOrder.getPartNo());
        }
        //物料+生产版本获取bom信息
        ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesProductVersion.getProductVersion(), "bomVersion", ddlPackBean);
        List<MesBom> mesBoms = mesBomRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(mesBoms)) {
            MesException.throwMesBusiException("物料【%s】生产版本【%s】对应bom信息不存在",
                    mesWorkOrder.getPartNo(), mesProductVersion.getProductVersion());
        }
        //报工类型
        if (MesExtEnumUtil.REPORT_TYPE.REPORT.getValue() == mesWorkOrder.getReportType()) {
            mesWorkOrder.setReportedQty(MathOperation.add(mesWorkOrder.getNum(), mesWorkOrder.getReportedQty()));
        } else {
            //冲销数量大于工单完成数量报错
            if (mesWorkOrder.getNum() > mesWorkOrder.getReportedQty()) {
                MesException.throwMesBusiException("冲销数量【%s】大于工单完成数量【%s】,不允许冲销",
                        mesWorkOrder.getNum(), mesWorkOrder.getReportedQty());
            }
            //报工调整数量加-
            mesWorkOrder.setAdjustQty((MathOperation.add(mesWorkOrder.getNum(), mesWorkOrder.getAdjustQty())));
            //已汇报数量减一
            mesWorkOrder.setReportedQty(MathOperation.sub(mesWorkOrder.getReportedQty(), mesWorkOrder.getNum()));
        }
        //更新工单状态
        double unCompleteQty = MathOperation.sub(mesWorkOrder.getQty(), mesWorkOrder.getReportedQty());
        mesWorkOrder.setUnCompleteQty(unCompleteQty > 0 ? unCompleteQty : 0);
        if (mesWorkOrder.getReportedQty() > mesWorkOrder.getQty()) {
            MesException.throwMesBusiException("工单报工数量【%s】大于工单数量【%s】,不允许报工",
                    mesWorkOrder.getReportedQty(), mesWorkOrder.getQty());
        } else if (Objects.equals(mesWorkOrder.getReportedQty(), mesWorkOrder.getQty())) {
            mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
        } else {
            mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.process.getValue());
        }
        update(mesWorkOrder);

        //生成条码
        GenSerialNoModel genSerialNoModel = new GenSerialNoModel("REPORT_SN");
        genSerialNoModel.setPartNo(mesWorkOrder.getPartNo());
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, mesWorkOrder.getCreateUser(), mesWorkOrder.getOrganizeCode(), mesWorkOrder.getNum()).getResultList();
        //查询产线信息
        ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesWorkCenter)) {
            MesException.throwMesBusiException("产线【%s】不存在", mesWorkOrder.getWorkCenterCode());
        }
        //保存数据
        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
        MesProductOffLine newMesProductOffLine;
        for (String sn : resultList) {
            for (MesBom mesBom : mesBoms) {
                newMesProductOffLine = new MesProductOffLine();
                newMesProductOffLine.setReportPartNo(mesWorkOrder.getPartNo());
                newMesProductOffLine.setReportPartNameRdd(mesWorkOrder.getPartNameRdd());
                newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
                newMesProductOffLine.setItemPartNameRdd(mesBom.getItemPartName());
                newMesProductOffLine.setItemQty(mesBom.getItemQty());
                newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
                newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
                newMesProductOffLine.setQty(1d);
                newMesProductOffLine.setReportSn(sn);
                newMesProductOffLine.setBomVersion(mesBom.getBomVersion());
                newMesProductOffLine.setSerialNumber(sn);
                newMesProductOffLine.setUnit(mesBom.getUnit());
                newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
                newMesProductOffLine.setWorkOrderNo(mesWorkOrder.getOrderNo());
                newMesProductOffLine.setWorkCenterCode(mesWorkOrder.getWorkCenterCode());
                newMesProductOffLine.setWorkCellCode(mesWorkOrder.getWorkCellCode());
                newMesProductOffLine.setReportType(mesWorkOrder.getReportType());
                newMesProductOffLine.setSapWorkCenter(mesWorkCenter.getSapWorkCenter());
                newMesProductOffLine.setOrganizeCode(mesWorkOrder.getOrganizeCode());

                ConvertBean.serviceModelInitialize(newMesProductOffLine, mesWorkOrder.getCreateUser());
                mesProductOffLineList.add(newMesProductOffLine);
            }
        }
        mesProductOffLineService.insertBatch(mesProductOffLineList);
    }

    @Override
    public List<MesWorkOrder> sumMesWorkOrderQtyToSap(String organizeCode) {
        List<Integer> statusList = new ArrayList<>();
        statusList.add(MesExtEnumUtil.ORDER_STATUS.CREATE.getValue());
//        statusList.add(MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue());
        //查询汇总工单数量
        return mesWorkOrderDao.sumMesWorkOrderQtyToSap(organizeCode, statusList);
    }
}
