package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.mes.api.iservice.busi.IExcelImportService;
import cn.estsh.i3plus.mes.apiservice.serviceimpl.busi.CommonService;
import cn.estsh.i3plus.mes.apiservice.util.DateUtil;
import cn.estsh.i3plus.mes.apiservice.util.MesCommonUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.util.BaseRedis;
import cn.estsh.i3plus.pojo.mes.bean.MesItemPartErrorPrevention;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.model.ExcelImportErrorModel;
import cn.estsh.i3plus.pojo.mes.model.ExcelImportResultModel;
import cn.estsh.i3plus.pojo.mes.repository.MesItemPartErrorPreventionRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartRepository;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFDataFormat;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.ss.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description:
 * @CreateDate 2022/6/27 17:28
 * @Author 尹肖
 */
@Service
@Slf4j
public class MesItemPartErrorPreventionExcelService implements IExcelImportService {

    @Autowired
    private CommonService commonService;
    @Autowired
    private MesPartRepository partRepository;
    @Autowired
    private MesItemPartErrorPreventionRepository itemPartErrorPreventionRepository;

    @Override
    public ExcelImportResultModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {

        //读取表格
        ExcelImportResultModel excelImportResultModel = this.sheetExtractMesStationBom(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesItemPartErrorPrevention(excelImportResultModel);
        return excelImportResultModel;
//        return null;
    }

    /**
     * 工位BOM表导入-读取表格
     *
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultModel sheetExtractMesStationBom(Sheet sheetAt, String organizeCode, String userName) {

        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesStationBom集合
        List<MesItemPartErrorPrevention> itemPartErrorPreventions = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorModel> excelImportErrorModels = new ArrayList<>();

        // 查询物料表
        Map<String, Object> partCodeEntityMap;
        try {
            partCodeEntityMap = MesCommonUtil.getCodeEntityMap(partRepository, "partNo", organizeCode, "物料表");
        } catch (Exception e) {
            partCodeEntityMap = null;
        }

        //查询关键件条码校验规则表
        List<MesItemPartErrorPrevention> itemPartErrorPreventionList = itemPartErrorPreventionRepository.findByProperty(new String[]{MesConstWords.ORGANIZE_CODE, MesConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesItemPartErrorPrevention>> preventionMap =
                itemPartErrorPreventionList.stream().collect(Collectors.groupingBy(k -> k.getPreventionObjectCode() + "&" + k.getPartNo()));


        Map<String, String> map = new HashMap<>();

        for (int i = (sheetAt.getFirstRowNum() + 1); i < totalNumberOfRows; i++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //防错对象代码
                String preventionObjectCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();

                //防错代码
                String preventionCode = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    preventionCode = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //零件号
                String partNo = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    partNo = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //批次号
                String lot = null;
                try {
                    row.getCell(3).setCellType(CellType.STRING);
                    lot = row.getCell(3, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }


                if (!map.containsKey(preventionObjectCode + "&" + partNo)) {
                    map.put(preventionObjectCode + "&" + partNo, String.valueOf(rowNum));
                } else {
                    errorNum++;
                    cellNum += "A;C;";
                    errorInfo += "第A列数据:" + preventionObjectCode + ",第C列数据:" + partNo;
                }

                //校验物料号
                if (org.springframework.util.StringUtils.isEmpty(partNo)) {
                    errorNum++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                } else {
                    if (CollectionUtils.isEmpty(partCodeEntityMap) ||
                            (!CollectionUtils.isEmpty(partCodeEntityMap) && !partCodeEntityMap.containsKey(partNo))) {
                        errorNum++;
                        cellNum += "C;";
                        errorInfo += "第C列数据无效;";
                    }
                }


                //没有错误的时候，封装MesStationBom
                if (errorNum == 0) {

                    MesItemPartErrorPrevention itemPartErrorPrevention;
                    if (preventionMap.containsKey(preventionObjectCode + "&" + partNo)) {
                        itemPartErrorPrevention = preventionMap.get(partNo).get(0);
                        ConvertBean.serviceModelUpdate(itemPartErrorPrevention, userName);
                    } else {
                        itemPartErrorPrevention = new MesItemPartErrorPrevention();
                        ConvertBean.serviceModelInitialize(itemPartErrorPrevention, userName);
                    }
                    itemPartErrorPrevention.setPartNo(partNo);
                    itemPartErrorPrevention.setOrganizeCode(organizeCode);
                    MesPart part = (MesPart) partCodeEntityMap.get(partNo);
                    itemPartErrorPrevention.setPartName(part.getPartName());
                    itemPartErrorPrevention.setLot(lot);
                    itemPartErrorPrevention.setPreventionObjectCode(preventionObjectCode);
                    itemPartErrorPrevention.setPreventionCode(preventionCode);
                    itemPartErrorPreventions.add(itemPartErrorPrevention);
                    successRowNum++;
                } else {
                    //封装错误行信息ExcelImportErrorModel
                    excelImportErrorModels = commonService.getExcelImportErrorModels(excelImportErrorModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum++;
                }

            }
        }

        //校验EXCEL数据
        commonService.checkExcelData(failRowNum, successRowNum, errorRows);
        //封装返回结果
        ExcelImportResultModel excelImportResultModel = commonService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : itemPartErrorPreventions);
        return excelImportResultModel;
    }

    private void insertExcelMesItemPartErrorPrevention(ExcelImportResultModel excelImportResultModel) {
        if (excelImportResultModel == null) {
            return;
        }
        List<MesItemPartErrorPrevention> itemPartErrorPreventions = excelImportResultModel.getExcelList();
        if (CollectionUtils.isEmpty(itemPartErrorPreventions)) {
            return;
        }

        itemPartErrorPreventions.forEach(k -> {
            if (StringUtils.isEmpty(k.getId())) {
                itemPartErrorPreventionRepository.insert(k);
            } else {
                itemPartErrorPreventionRepository.update(k);
            }
        });
    }
}
